/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global popupController, pageController*/

(function() {
    var pageList = ["page-main", "page-contact"],
        addressbook,
        contacts;

    /**
     * Removes all child of the element.
     * @private
     * @param {Object} elm - The object to be emptied
     * @return {Object} The emptied element
     */
    function emptyElement(elm) {
        while (elm.firstChild) {
            elm.removeChild(elm.firstChild);
        }

        return elm;
    }

    /**
     * Handles the hardware key event.
     * @private
     * @param {Object} event - The hardware key event object
     */
    function keyEventHandler(event) {
        if (event.keyName === "back") {
            if (popupController.isActive() === true) {
                popupController.closePopup();
            } else if (pageController.isPageMain() || pageController.moveBackPage() === false) {
                try {
                    tizen.application.getCurrentApplication().exit();
                } catch (ignore) {}
            }
        }
    }

    /**
     * Shows the detailed information of the selected contact.
     * @private
     * @param {number} index - The index of the contact to be displayed
     */
    function showContact(index) {
        var objResult = emptyElement(document.querySelector("#detail-contact")),
            objTable,
            objRow,
            objCol,
            objImg,
            prop,
            data = {
                name: contacts[index].name.displayName,
                phone: ((contacts[index].phoneNumbers.length > 0) ? contacts[index].phoneNumbers[0].number : null),
                address: ((contacts[index].addresses.length > 0) ? contacts[index].addresses[0].streetAddress : null),
                email: ((contacts[index].emails.length > 0) ? contacts[index].emails[0].email : null),
                note: ((contacts[index].notes.length > 0) ? contacts[index].notes[0] : null),
                photo: contacts[index].photoURI || null
            };

        // Create a table to contain the information
        objTable = document.createElement("div");
        objTable.className = "contact-table";

        // Show each property of the contact with a separated row
        for (prop in data) {
            if (data.hasOwnProperty(prop) && data[prop] !== null) {
                objRow = document.createElement("div");
                objRow.className = "contact-table-row";

                objCol = document.createElement("div");
                objCol.className = "type-detail";
                objCol.appendChild(document.createTextNode(prop));
                objRow.appendChild(objCol);

                objCol = document.createElement("div");
                objCol.className = "value-detail";
                if (prop !== "photo") {
                    objCol.appendChild(document.createTextNode(data[prop]));
                } else {
                    // If the property is an address of photo, show photo instead of the address itself
                    objImg = document.createElement("img");
                    objImg.className = "photo-detail";
                    objImg.src = data[prop];
                    objCol.appendChild(objImg);
                }
                objRow.appendChild(objCol);

                objTable.appendChild(objRow);
            }
        }

        // Append the table to the result page
        objResult.appendChild(objTable);

        // Show the contact page
        pageController.movePage("page-contact");
    }

    /**
     * Creates the callback function to handle the click event of each element in contact list.
     * @private
     * @param {number} index - The index of the contact
     */
    function createCallback(index) {
        var retFunc = function() {
            var data = {
                index: index
            };

            showContact(data.index);
        };

        return retFunc;
    }

    /**
     * Shows the contact list to be displayed in the main page.
     * @private
     * @param {Object[]} dataArray - The array of the contacts data
     */
    function showListView(dataArray) {
        var objResult = emptyElement(document.querySelector("#detail-main")),
            objTable,
            objRow,
            objCol,
            i,
            prop;

        // Create a table to contain the information
        objTable = document.createElement("div");
        objTable.className = "result-table";

        if (dataArray.length > 0) {
            for (i = 0; i < dataArray.length; i++) {
                objRow = document.createElement("div");

                objRow.className = "result-table-row";
                // Show each property of the with a separated column
                for (prop in dataArray[i]) {
                    if (dataArray[i].hasOwnProperty(prop)) {
                        objCol = document.createElement("div");
                        objCol.className = prop + "-detail";
                        if (dataArray[i][prop].hasOwnProperty("data")) {
                            objCol.appendChild(document.createTextNode(dataArray[i][prop].data));
                        }
                        if (dataArray[i][prop].hasOwnProperty("cb")) {
                            objCol.addEventListener("click", dataArray[i][prop].cb);
                        }

                        objRow.appendChild(objCol);
                    }
                }

                objTable.appendChild(objRow);
            }
        } else {
            // Shows "No data" text if there is no contact data.
            objRow = document.createElement("div");
            objRow.className = "result-table-error";
            objRow.appendChild(document.createTextNode("No Data"));

            objTable.appendChild(objRow);
        }

        // Append the table to the result page
        objResult.appendChild(objTable);
    }

    /**
     * Sets the default style of elements.
     * @private
     */
    function setDefaultViews() {
        var defaultErrorMessage = document.querySelector("#message-error"),
            dataContacts = [],
            i;

        for (i = 0; i < contacts.length; i++) {
            dataContacts.push({
                name: {
                    data: contacts[i].name.displayName,
                    cb: createCallback(i)
                }
            });
        }
        showListView(dataContacts);

        defaultErrorMessage.style.display = "none";
    }

    /**
     * Sets the default value to the variables and application environment.
     * @private
     */
    function setDefaultVariables() {
        var i;

        popupController.createPopup("div-popup", "div-popup-detail", "div-popup-footer");
        // Add all pages in pageList to pageController
        for (i = 0; i < pageList.length; i++) {
            pageController.addPage(pageList[i]);
        }

        if (window.tizen && tizen.contact && tizen.contact.getAddressBooks) {
            try {
                addressbook = tizen.contact.getDefaultAddressBook();
                addressbook.find(function(result) {
                    contacts = result;
                    setDefaultViews();
                }, function() {
                    popupController.showPopup("Failed to get contacts", "OK");
                });
            } catch (err) {
                return false;
            }

            return true;
        }

        return false;
    }

    /**
     * Sets the default event handlers to the events.
     * @private
     */
    function setDefaultEvents() {
        // Add hardware key event listener
        document.addEventListener("tizenhwkey", keyEventHandler);

        return true;
    }

    /**
     * Initializes the application.
     * @private
     */
    function init() {
        if (setDefaultEvents() && setDefaultVariables()) {
            console.log("App: init");
            popupController.showPopup("Success", "OK");
        } else {
            popupController.showPopup("getContactList failed.", "OK");
        }
    }

    window.onload = init;
}());